﻿using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System;

namespace CKPoolSet
{
    /// <summary>
    /// 数据连接
    /// </summary>
    public class DatabaseLgc
    {

        private static string connectionString = "Provider=Microsoft.Jet.Oledb.4.0;data source=" + AppDomain.CurrentDomain.BaseDirectory + "\\" + System.Configuration.ConfigurationManager.AppSettings["PoolSetDatabase"]; // TODO: 初始化为适当的值

        /// <summary>
        /// 设置数据库连接串
        /// </summary>
        public static string ConnectionString
        {
            get { return DatabaseLgc.connectionString; }
            set { DatabaseLgc.connectionString = value; }
        }

        private OleDbConnection cn = null;

        /// <summary>
        /// 打开数据库
        /// </summary>
        /// <returns></returns>
        private void OpenData()
        {
            if (cn == null || cn.State == ConnectionState.Closed)
            {
                cn = new OleDbConnection(connectionString);
                cn.Open();
            }
        }

        /// <summary>
        ///关闭数据库
        /// </summary>
        /// <returns></returns>
        private void CloseData()
        {
            if (cn != null)
            {
                cn.Close();
                cn.Dispose();
            }
        }

        /// <summary>
        /// 返回数据集
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public DataTable GetTable(string sql, params object[] parameters)
        {
            OpenData();
            OleDbTransaction dbt = cn.BeginTransaction();
            DataTable userTbl = new DataTable();
            OleDbCommand cmd = new OleDbCommand(sql, cn, dbt);
            for (int i = 0; i < parameters.Length; i++)
            {
                cmd.Parameters.Add(parameters[i]);
            }
            OleDbDataAdapter da = new OleDbDataAdapter(cmd);
            da.Fill(userTbl);
            CloseData();
            userTbl.RemotingFormat = SerializationFormat.Binary;
            return userTbl;
        }

        /// <summary>
        /// 插入语句
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public bool ExecSql(string sql)
        {
            OpenData();
            OleDbTransaction dbt = cn.BeginTransaction();
            int a = 0;
            try
            {
                OleDbCommand cmd = new OleDbCommand(sql, cn, dbt);
                a = cmd.ExecuteNonQuery();
                dbt.Commit();
            }
            catch (Exception exp)
            {
                dbt.Rollback();
                throw exp;
            }
            CloseData();
            if (a > 0)
            {
                return true;
            }
            return false;
        }
        /// <summary>
        /// 插入语句
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public bool ExecSqls(List<string> sql)
        {
            OpenData();
            OleDbTransaction dbt = cn.BeginTransaction();
            int a = 0;
            try
            {
                for (int i = 0; i < sql.Count; i++)
                {
                    OleDbCommand cmd = new OleDbCommand(sql[i], cn, dbt);
                    a = cmd.ExecuteNonQuery();
                }
                dbt.Commit();
            }
            catch (Exception exp)
            {
                dbt.Rollback();
                CloseData();
                throw exp;
            }
            CloseData();
            if (a > 0)
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="sqlParameter"></param>
        /// <returns></returns>
        public bool ExecSql(string sql, List<OleDbParameter> sqlParameter)
        {
            OpenData();
            OleDbTransaction dbt = cn.BeginTransaction();
            int a = 0;
            try
            {
                OleDbCommand cmd = new OleDbCommand(sql, cn, dbt);
                for (int i = 0; i < sqlParameter.Count; i++)
                {
                    cmd.Parameters.Add(sqlParameter[i]);
                }
                a = cmd.ExecuteNonQuery();
                dbt.Commit();
            }
            catch (Exception exp)
            {
                dbt.Rollback();
                CloseData();
                throw exp;
            }
            CloseData();
            if (a > 0)
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tranLst"></param>
        /// <returns></returns>
        public bool ExecSqls(List<TransEntity> tranLst)
        {
            OpenData();
            OleDbTransaction dbt = cn.BeginTransaction();
            int a = 0;
            try
            {
                foreach (var item in tranLst)
                {
                    OleDbCommand cmd = new OleDbCommand(item.Sql, cn, dbt);
                    if (item.SqlParameters != null)
                    {
                        for (int i = 0; i < item.SqlParameters.Count; i++)
                        {
                            cmd.Parameters.Add(item.SqlParameters[i]);
                        }
                    }
                    a = cmd.ExecuteNonQuery();
                }
                dbt.Commit();
            }
            catch(Exception exp)
            {
                dbt.Rollback();
                CloseData();
                throw exp;
            }
            CloseData();
            if (a > 0)
            {
                return true;
            }
            return false;
        }
    }
}
