﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.CodeDom.Compiler;
using CKRule;
using System.Data.OleDb;
using CKPoolSet;

namespace Salary
{
    /// <summary>
    /// 客户规则池测试
    /// </summary>
    partial class FrmRulePoolSet : Form
    {
        BindingList<CustPoolCond> _customRuleList;
        EditState _curEditState;
        PackageDef _curPackage;
        /// <summary>
        /// 规则实例的名称
        /// </summary>
        public const string RuleInstName = "工资计算";

        /// <summary>
        /// 初始化
        /// </summary>
        public FrmRulePoolSet()
        {
            InitializeComponent();
        }

        private void FrmRulePoolSet_Load(object sender, EventArgs e)
        {
            _curPackage = new RuleFacade().GetPkgInst(RuleInstName);
            if (_curPackage == null)
            {
                MessageBox.Show("加载规则包失败");
                return;
            }
            dgvLst.AutoGenerateColumns = false;
            InitListView();
            Query();
        }


        /// <summary>
        /// 初始化ListView
        /// </summary>
        private void InitListView()
        {
            //lvTips.View = View.Tile;
            foreach (var item in _curPackage.RuleLib.PoolRules.Keywords)
            {
                ListViewItem _lvi = new ListViewItem();
                _lvi.Text = item.Name;
                _lvi.Tag = item;
                //_lvi.ImageKey = "PicProperty";
                _lvi.Group = lvTips.Groups[0];
                lvTips.Items.Add(_lvi);
            }
            foreach (var item in _curPackage.RuleLib.PoolRules.LgcOperaWords)
            {
                ListViewItem _lvi = new ListViewItem();
                _lvi.Text = item.Name;
                _lvi.Tag = item;
                //_lvi.ImageKey = "PicMethod";                
                _lvi.Group = lvTips.Groups[1];
                lvTips.Items.Add(_lvi);
            }
            foreach (var item in _curPackage.RuleLib.PoolRules.ConclusionWords)
            {
                ListViewItem _lvi = new ListViewItem();
                _lvi.Text = item.Name;
                _lvi.Tag = item;// new List<CallDef>() { item };
                //_lvi.ImageKey = "PicMethod";
                _lvi.Group = lvTips.Groups[2];
                lvTips.Items.Add(_lvi);
            }
            lvTips.MouseDoubleClick += new MouseEventHandler(lvTips_MouseDoubleClick);
        }

        void lvTips_MouseDoubleClick(object sender, MouseEventArgs e)
        {//ListView控件点击时
            if (lvTips.SelectedItems.Count == 0) return;
            var _item = lvTips.SelectedItems[0];
            var _kwdef = _item.Tag as KeywordDef;
            if (_kwdef != null)
            {
                SetEditorText(_kwdef.Name);
                txtReturnType.Text = _kwdef.ReturnType;
                txtDescription.Text = _kwdef.Description;
                txtMethodName.Text = _kwdef.Name;
                dgvParameters.DataSource = new BindingList<ParamDef>();
                tsbOK.Enabled = false;
                return;
            }
            var _callDef = _item.Tag as CallDef;
            if (_callDef != null)
            {
                txtReturnType.Text = _callDef.ReturnType;
                txtDescription.Text = _callDef.Description;
                txtMethodName.Text = _callDef.Name;
                dgvParameters.DataSource = new BindingList<ParamDef>(_callDef.Parameters);
                tsbOK.Enabled = true;
                return;
            }
        }

        /// <summary>
        /// 设置编辑器的字符
        /// </summary>
        /// <param name="text"></param>
        private void SetEditorText(string text)
        {
            var _curTextBox = txtContent.Text;
            if (_curTextBox == null) return;
            txtContent.SelectedText = text;
        }

        DatabaseLgc _access = new DatabaseLgc();

        private void Query()
        {
            var _datas = _access.GetTable("select * from poolset").ToList<CustPoolCond>();
            _customRuleList = new BindingList<CustPoolCond>(_datas);
            if (_datas.Count == 0)
            {
                string _items = "实发基本工资,平时加班费用,周末加班合计费用,节假日加班费用,平时顶班费用,周末顶班费用,节假日顶班费用,实发高温津贴,实发学历补贴,奖金数,实发气味津贴,实发粉尘津贴,事假扣除,病假扣除,旷工扣除,实发工资";
                List<string> itemLst = _items.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries).ToList();
                foreach (var item in itemLst)
                {
                    _customRuleList.Add(new CustPoolCond() { Name = item });
                }
            }
            dgvLst.DataSource = _customRuleList;
            _curEditState = EditState.None;
            SetCtrlState();
        }

        private void SetCtrlState()
        {
            switch (_curEditState)
            {
                case EditState.Add:
                case EditState.Edit:
                    dgvLst.Enabled = false;
                    tsbCancelEdit.Enabled = true;
                    tsbSubmit.Enabled = true;
                    pnlEditDtl.Enabled = true;
                    break;
                case EditState.None:
                    dgvLst.Enabled = true;
                    tsbCancelEdit.Enabled = false;
                    tsbSubmit.Enabled = false;
                    pnlEditDtl.Enabled = false;
                    break;
            }
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            Query();
        }

        private void tsbNewTable_Click(object sender, EventArgs e)
        {
            pnlEdit.Enabled = true;
            _curEditState = EditState.Add;
            SetCtrlState();
            var _inst = _customRuleList.AddNew();
            dgvLst.ClearSelection();
            foreach (DataGridViewRow item in dgvLst.Rows)
            {
                var _curDef = item.DataBoundItem as CustPoolCond;
                if (_curDef == _inst)
                {
                    item.Selected = true;
                }
            }
            SetPropDef(_inst);
            dgvLst.Refresh();
        }

        private void tsbDelTable_Click(object sender, EventArgs e)
        {
            if (dgvLst.SelectedRows.Count == 0) return;
            var _selRow = dgvLst.SelectedRows[0];
            if (_selRow == null) return;
            //HasChanged = true;
            var _ruleDef = _selRow.DataBoundItem as CustPoolCond;
            _customRuleList.Remove(_ruleDef);
            //_curPackage.RuleLib.TestCaseDef.CustomRules.Remove(_ruleDef);
            dgvLst.Refresh();
            _curEditState = EditState.None;
            SetCtrlState();
        }

        private void tsbExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void dgvLst_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (dgvLst.SelectedRows.Count == 0) return;
            var _selRow = dgvLst.SelectedRows[0];
            if (_selRow == null) return;
            pnlEdit.Enabled = true;
            var _ruleDef = _selRow.DataBoundItem as CustPoolCond;
            SetPropDef(_ruleDef);
            _curEditState = EditState.Edit;
            SetCtrlState();
        }

        private void SetPropDef(CustPoolCond _ruleDef)
        {
            txtPropName.Text = _ruleDef.Name;
            txtContent.Text = _ruleDef.CalcCode;
        }

        private void tsbSubmit_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtPropName.Text))
            {
                MessageBox.Show("提示_设置的结点名称不为能空");
                return;
            }
            if (dgvLst.SelectedRows.Count == 0) return;
            var _selRow = dgvLst.SelectedRows[0];
            if (_selRow == null) return;
            var _ruleDef = _selRow.DataBoundItem as CustPoolCond;
            _ruleDef.Name = txtPropName.Text;
            _ruleDef.CalcCode = txtContent.Text;
            dgvLst.Refresh();
            _curEditState = EditState.None;
            SetCtrlState();
        }

        private void tsbOK_Click(object sender, EventArgs e)
        {
            dgvParameters.CommitEdit(DataGridViewDataErrorContexts.Commit);
            var _list = dgvParameters.DataSource as BindingList<ParamDef>;
            if (_list == null) return;
            var _paramStr = string.Join(",", (from x1 in _list
                                              select x1.Name));
            SetEditorText(txtMethodName.Text + "(" + _paramStr + ")");
        }

        private void tsbTest_Click(object sender, EventArgs e)
        {
            Test(false);
        }

        private void tsbTryRun_Click(object sender, EventArgs e)
        {
            Test(true);
        }

        private void Test(bool tryRun)
        {
            try
            {
                if (dgvLst.SelectedRows.Count == 0) return;
                var _selRow = dgvLst.SelectedRows[0];
                if (_selRow == null) return;
                var _poolCond = _selRow.DataBoundItem as CustPoolCond;
                if (_poolCond == null) return;
                var _ruleDef = new CustUnstructPropDef();
                _ruleDef.ThenCodeIsScript = false;
                _ruleDef.PropName = "Get" + _poolCond.Name;
                _ruleDef.CodeStyle = CodeStyleDef.SetStyle;
                _ruleDef.ThenCode = "结果." + _poolCond.Name + "=" + Environment.NewLine +
                    " " + _poolCond.CalcCode + " ;" + Environment.NewLine + "";
                new RuleFacade().TestPool(RuleInstName, _ruleDef);
                MessageBox.Show("测试语法成功");
            }
            catch (Exception exp)
            {
                MessageBox.Show("测试语法失败" + exp.Message);
            }
        }

        private void tsbCancelEdit_Click(object sender, EventArgs e)
        {
            _curEditState = EditState.None;
            SetCtrlState();
        }

        private void tsbSave_Click(object sender, EventArgs e)
        {

            List<TransEntity> _tranLst = new List<TransEntity>();
            _tranLst.Add(new TransEntity() { Sql = "delete * from poolset" });
            foreach (var item in _customRuleList)
            {
                var _trans = new TransEntity();
                _trans.Sql = @"insert into poolset(SIndex,Name,CalcCode)values(@SIndex,@Name,@CalcCode)";
                _trans.SqlParameters.Add(new OleDbParameter("@SIndex", item.SIndex));
                _trans.SqlParameters.Add(new OleDbParameter("@Name", item.Name));
                _trans.SqlParameters.Add(new OleDbParameter("@CalcCode", item.CalcCode + string.Empty));
                _tranLst.Add(_trans);
            }
            try
            {
                _access.ExecSqls(_tranLst);
                new RuleFacade().Refresh(RuleInstName);
                MessageBox.Show("保存成功");
                Query();
            }
            catch (Exception exp)
            {
                MessageBox.Show("保存错误：" + exp.Message);
            }
        }
    }
}
